#       short2long - generate database file with long attributes
#
# Copyright (c) 1993, 1994, 1995, 1996, 1997, 1998, 1999 by RIPE NCC
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# $Id: short2long.pl,v 2.4 1999/10/06 16:41:20 marek Exp $
#
#       $RCSfile: short2long.pl,v $
#       $Revision: 2.4 $
#       $Author: marek $
#       $Date: 1999/10/06 16:41:20 $

# Syntax:
#
# short2long <InputFile >Outputfile

require "rconf.pl";

local($conffile) = $ENV{"RIPEDBCNF"};
$conffile = $RIPEConfig{DEFCONFIG} unless $conffile;

&rconf($conffile);

#
# and here we go...

local($attribute);
local(@output)=();

while (<STDIN>) {
   
   if (s/^\*(..)\: *//) {
   
      $attribute=$ATTL{$1};
      
      if ($attribute) {
      
         push(@output, $attribute, ":", " " x ($MAXATTRIBUTELENGTH-length($attribute)+1), $_);
         
      }
      else {
      
         push(@output, "*" , $1, ":", $_);
         
      }
      
   }
   else {
   
      push(@output, $_);
   
   }
   
   if (scalar(@output)>$MAXLISTLENGTH) {
      print STDOUT @output;
      @output=();
   }
   
}

print STDOUT @output;

# end of program
